@@ -1,7 +1,7 @@
 # flake8: noqa: DJ01
 import hmac
 from datetime import date, datetime, timedelta
-from typing import Iterable, List, Optional, Sequence, Union
+from typing import Any, Iterable, List, Optional, Sequence, Union
 from urllib.parse import urlparse
 
 from django.conf import settings
@@ -14,7 +14,7 @@ from django.contrib.sites.models import Site
 from django.core.exceptions import ValidationError
 from django.core.validators import MaxValueValidator
 from django.db import models, transaction
-from django.db.models import QuerySet
+from django.db.models import Q, QuerySet
 from django.db.models.signals import m2m_changed
 from django.dispatch import receiver
 from django.forms.widgets import Media
@@ -28,7 +28,12 @@ import jsonstore
 from cachalot.api import cachalot_disabled
 from cache_memoize import cache_memoize
 from django_celery_results.models import TaskResult
+from django_cte import CTEQuerySet, With
 from dynamic_preferences.models import PerInstancePreferenceModel
+from invitations import signals
+from invitations.adapters import get_invitations_adapter
+from invitations.base_invitation import AbstractBaseInvitation
+from invitations.models import Invitation
 from model_utils import FieldTracker
 from model_utils.models import TimeStampedModel
 from oauth2_provider.models import (
@@ -40,7 +45,6 @@ from oauth2_provider.models import (
 )
 from phonenumber_field.modelfields import PhoneNumberField
 from polymorphic.models import PolymorphicModel
-from templated_email import send_templated_mail
 
 from aleksis.core.data_checks import BrokenDashboardWidgetDataCheck, DataCheck, DataCheckRegistry
 
@@ -59,7 +63,8 @@ from .mixins import (
     SchoolTermRelatedExtensibleModel,
 )
 from .tasks import send_notification
-from .util.core_helpers import get_site_preferences, now_tomorrow
+from .util.core_helpers import generate_random_code, get_site_preferences, now_tomorrow
+from .util.email import send_email
 from .util.model_helpers import ICONS
 
 FIELD_CHOICES = (
@@ -150,6 +155,7 @@ class Person(ExtensibleModel):
             ("view_address", _("Can view address")),
             ("view_contact_details", _("Can view contact details")),
             ("view_photo", _("Can view photo")),
+            ("view_avatar", _("Can view avatar image")),
             ("view_person_groups", _("Can view persons groups")),
             ("view_personal_details", _("Can view personal details")),
         )
@@ -171,7 +177,6 @@ class Person(ExtensibleModel):
         related_name="person",
         verbose_name=_("Linked user"),
     )
-    is_active = models.BooleanField(verbose_name=_("Is person active?"), default=True)
 
     first_name = models.CharField(verbose_name=_("First name"), max_length=255)
     last_name = models.CharField(verbose_name=_("Last name"), max_length=255)
@@ -194,12 +199,24 @@ class Person(ExtensibleModel):
     email = models.EmailField(verbose_name=_("E-mail address"), blank=True)
 
     date_of_birth = models.DateField(verbose_name=_("Date of birth"), blank=True, null=True)
-    place_of_birth = models.CharField(
-        verbose_name=_("Place of birth"), max_length=255, blank=True, null=True
-    )
+    place_of_birth = models.CharField(verbose_name=_("Place of birth"), max_length=255, blank=True)
     sex = models.CharField(verbose_name=_("Sex"), max_length=1, choices=SEX_CHOICES, blank=True)
 
-    photo = models.ImageField(verbose_name=_("Photo"), blank=True, null=True)
+    photo = models.ImageField(
+        verbose_name=_("Photo"),
+        blank=True,
+        null=True,
+        help_text=_(
+            "This is an official photo, used for official documents and for internal use cases."
+        ),
+    )
+
+    avatar = models.ImageField(
+        verbose_name=_("Display picture / Avatar"),
+        blank=True,
+        null=True,
+        help_text=_("This is a picture or an avatar for public display."),
+    )
 
     guardians = models.ManyToManyField(
         "self",
@@ -295,8 +312,28 @@ class Person(ExtensibleModel):
         """Return the count of unread notifications for this person."""
         return self.unread_notifications.count()
 
+    @property
+    def initials(self):
+        return f"{self.first_name[0]}{self.last_name[0]}".upper()
+
     user_info_tracker = FieldTracker(fields=("first_name", "last_name", "email"))
 
+    @property
+    def member_of_recursive(self) -> QuerySet:
+        """Get all groups this person is a member of, recursively."""
+        q = self.member_of
+        for group in q.all():
+            q = q.union(group.parent_groups_recursive)
+        return q
+
+    @property
+    def owner_of_recursive(self) -> QuerySet:
+        """Get all groups this person is a member of, recursively."""
+        q = self.owner_of
+        for group in q.all():
+            q = q.union(group.child_groups_recursive)
+        return q
+
     def save(self, *args, **kwargs):
         # Determine all fields that were changed since last load
         dirty = self.pk is None or bool(self.user_info_tracker.changed())
@@ -349,7 +386,7 @@ class Person(ExtensibleModel):
         recipients = recipients or [
             get_site_preferences()["account__person_change_notification_contact"]
         ]
-        send_templated_mail(
+        send_email(
             template_name="person_changed",
             from_email=self.mail_sender_via,
             headers={
@@ -460,6 +497,21 @@ class Group(SchoolTermRelatedExtensibleModel):
         AdditionalField, verbose_name=_("Additional fields"), blank=True
     )
 
+    photo = models.ImageField(
+        verbose_name=_("Photo"),
+        blank=True,
+        null=True,
+        help_text=_(
+            "This is an official photo, used for official documents and for internal use cases."
+        ),
+    )
+    avatar = models.ImageField(
+        verbose_name=_("Display picture / Avatar"),
+        blank=True,
+        null=True,
+        help_text=_("This is a picture or an avatar for public display."),
+    )
+
     def get_absolute_url(self) -> str:
         return reverse("group_by_id", args=[self.id])
 
@@ -484,6 +536,50 @@ class Group(SchoolTermRelatedExtensibleModel):
 
         return stats
 
+    @property
+    def parent_groups_recursive(self) -> CTEQuerySet:
+        """Get all parent groups recursively."""
+
+        def _make_cte(cte):
+            Through = self.parent_groups.through
+            return (
+                Through.objects.values("to_group_id")
+                .filter(from_group=self)
+                .union(cte.join(Through, from_group=cte.col.to_group_id), all=True)
+            )
+
+        cte = With.recursive(_make_cte)
+        return cte.join(Group, id=cte.col.to_group_id).with_cte(cte)
+
+    @property
+    def child_groups_recursive(self) -> CTEQuerySet:
+        """Get all child groups recursively."""
+
+        def _make_cte(cte):
+            Through = self.child_groups.through
+            return (
+                Through.objects.values("from_group_id")
+                .filter(to_group=self)
+                .union(cte.join(Through, to_group=cte.col.from_group_id), all=True)
+            )
+
+        cte = With.recursive(_make_cte)
+        return cte.join(Group, id=cte.col.from_group_id).with_cte(cte)
+
+    @property
+    def members_recursive(self) -> QuerySet:
+        """Get all members of this group and its child groups."""
+        return Person.objects.filter(
+            Q(member_of=self) | Q(member_of__in=self.child_groups_recursive)
+        )
+
+    @property
+    def owners_recursive(self) -> QuerySet:
+        """Get all ownerss of this group and its parent groups."""
+        return Person.objects.filter(
+            Q(owner_of=self) | Q(owner_of__in=self.parent_groups_recursive)
+        )
+
     def __str__(self) -> str:
         if self.school_term:
             return f"{self.name} ({self.short_name}) ({self.school_term})"
@@ -764,40 +860,7 @@ class AnnouncementRecipient(ExtensibleModel):
 
 
 class DashboardWidget(PolymorphicModel, PureDjangoModel):
-    """Base class for dashboard widgets on the index page.
-
-    To implement a widget, add a model that subclasses DashboardWidget, sets the template
-    and implements the get_context method to return a dictionary to be passed as context
-    to the template.
-
-    If your widget does not add any database fields, you should mark it as a proxy model.
-
-    You can provide a Media meta class with custom JS and CSS files which
-    will be added to html head.  For further information on media definition
-    see https://docs.djangoproject.com/en/3.0/topics/forms/media/
-
-    Example::
-
-      from django.forms.widgets import Media
-
-      from aleksis.core.models import DashboardWidget
-
-      class MyWidget(DashboardWidget):
-          template = "myapp/widget.html"
-
-          def get_context(self, request):
-              context = {"some_content": "foo"}
-              return context
-
-          class Meta:
-              proxy = True
-
-          media = Media(css={
-                  'all': ('pretty.css',)
-              },
-              js=('animations.js', 'actions.js')
-          )
-    """
+    """Base class for dashboard widgets on the index page."""
 
     objects = UninstallRenitentPolymorphicManager()
 
@@ -997,6 +1060,7 @@ class GlobalPermissions(GlobalPermissionModel):
             ("change_person_preferences", _("Can change person preferences")),
             ("change_group_preferences", _("Can change group preferences")),
             ("test_pdf", _("Can test PDF generation")),
+            ("invite", _("Can invite persons")),
         )
 
 
@@ -1062,6 +1126,22 @@ class DataCheckResult(ExtensibleModel):
         )
 
 
+class PersonInvitation(AbstractBaseInvitation, PureDjangoModel):
+    """Custom model for invitations to allow to generate invitations codes without email address."""
+
+    email = models.EmailField(verbose_name=_("E-Mail address"), blank=True)
+    person = models.ForeignKey(
+        Person, on_delete=models.CASCADE, blank=True, related_name="invitation", null=True
+    )
+
+    def __str__(self) -> str:
+        return f"{self.email} ({self.inviter})"
+
+    key_expired = Invitation.key_expired
+
+    send_invitation = Invitation.send_invitation
+
+
 class PDFFile(ExtensibleModel):
     """Link to a rendered PDF file."""
 
@@ -1113,12 +1193,52 @@ class TaskUserAssignment(ExtensibleModel):
         verbose_name_plural = _("Task user assignments")
 
 
+class UserAdditionalAttributes(models.Model, PureDjangoModel):
+    """Additional attributes for Django user accounts.
+
+    These attributes are explicitly linked to a User, not to a Person.
+    """
+
+    user = models.OneToOneField(
+        get_user_model(),
+        on_delete=models.CASCADE,
+        related_name="additional_attributes",
+        verbose_name=_("Linked user"),
+    )
+
+    attributes = models.JSONField(verbose_name=_("Additional attributes"), default=dict)
+
+    @classmethod
+    def get_user_attribute(
+        cls, username: str, attribute: str, default: Optional[Any] = None
+    ) -> Any:
+        """Get a user attribute for a user by name."""
+        try:
+            attributes = cls.objects.get(user__username=username)
+        except cls.DoesNotExist:
+            return default
+
+        return attributes.attributes.get(attribute, default)
+
+    @classmethod
+    def set_user_attribute(cls, username: str, attribute: str, value: Any):
+        """Set a user attribute for a user by name.
+
+        Raises DoesNotExist if a username for a non-existing Django user is passed.
+        """
+        user = get_user_model().objects.get(username=username)
+        attributes, __ = cls.objects.update_or_create(user=user)
+
+        attributes.attributes[attribute] = value
+        attributes.save()
+
+
 class OAuthApplication(AbstractApplication):
     """Modified OAuth application class that supports Grant Flows configured in preferences."""
 
     # Override grant types to make field optional
     authorization_grant_type = models.CharField(
-        max_length=32, choices=AbstractApplication.GRANT_TYPES, blank=True, null=True
+        max_length=32, choices=AbstractApplication.GRANT_TYPES, blank=True
     )
 
     # Optional list of alloewd scopes
@@ -1129,10 +1249,19 @@ class OAuthApplication(AbstractApplication):
         blank=True,
     )
 
+    icon = models.ImageField(
+        verbose_name=_("Icon"),
+        blank=True,
+        null=True,
+        help_text=_(
+            "This image will be shown as icon in the authorization flow. It should be squared."
+        ),
+    )
+
     def allows_grant_type(self, *grant_types: set[str]) -> bool:
         allowed_grants = get_site_preferences()["auth__oauth_allowed_grants"]
 
-        return bool(set(allowed_grants) & grant_types)
+        return bool(set(allowed_grants) & set(grant_types))
 
 
 class OAuthGrant(AbstractGrant):
