@@ -3,6 +3,7 @@ from django.forms import EmailField, ImageField, URLField
 from django.forms.widgets import SelectMultiple
 from django.utils.translation import gettext_lazy as _
 
+import pycountry
 from colorfield.widgets import ColorWidget
 from dynamic_preferences.preferences import Section
 from dynamic_preferences.types import (
@@ -16,6 +17,7 @@ from dynamic_preferences.types import (
 )
 from oauth2_provider.models import AbstractApplication
 
+from .mixins import PublicFilePreferenceMixin
 from .models import Group, Person
 from .registries import person_preferences_registry, site_preferences_registry
 from .util.notifications import get_notification_choices_lazy
@@ -38,8 +40,8 @@ class SiteTitle(StringPreference):
     section = general
     name = "title"
     default = "AlekSIS"
-    required = False
     verbose_name = _("Site title")
+    required = True
 
 
 @site_preferences_registry.register
@@ -60,9 +62,9 @@ class ColourPrimary(StringPreference):
     section = theme
     name = "primary"
     default = "#0d5eaf"
-    required = False
     verbose_name = _("Primary colour")
     widget = ColorWidget
+    required = True
 
 
 @site_preferences_registry.register
@@ -72,39 +74,42 @@ class ColourSecondary(StringPreference):
     section = theme
     name = "secondary"
     default = "#0d5eaf"
-    required = False
     verbose_name = _("Secondary colour")
     widget = ColorWidget
+    required = True
 
 
 @site_preferences_registry.register
-class Logo(FilePreference):
+class Logo(PublicFilePreferenceMixin, FilePreference):
     """Logo of your AlekSIS instance."""
 
     section = theme
     field_class = ImageField
     name = "logo"
     verbose_name = _("Logo")
+    required = False
 
 
 @site_preferences_registry.register
-class Favicon(FilePreference):
+class Favicon(PublicFilePreferenceMixin, FilePreference):
     """Favicon of your AlekSIS instance."""
 
     section = theme
     field_class = ImageField
     name = "favicon"
     verbose_name = _("Favicon")
+    required = False
 
 
 @site_preferences_registry.register
-class PWAIcon(FilePreference):
+class PWAIcon(PublicFilePreferenceMixin, FilePreference):
     """PWA-Icon of your AlekSIS instance."""
 
     section = theme
     field_class = ImageField
     name = "pwa_icon"
     verbose_name = _("PWA-Icon")
+    required = False
 
 
 @site_preferences_registry.register
@@ -114,8 +119,8 @@ class MailOutName(StringPreference):
     section = mail
     name = "name"
     default = "AlekSIS"
-    required = False
     verbose_name = _("Mail out name")
+    required = True
 
 
 @site_preferences_registry.register
@@ -125,9 +130,9 @@ class MailOut(StringPreference):
     section = mail
     name = "address"
     default = settings.DEFAULT_FROM_EMAIL
-    required = False
     verbose_name = _("Mail out address")
     field_class = EmailField
+    required = True
 
 
 @site_preferences_registry.register
@@ -161,12 +166,12 @@ class AdressingNameFormat(ChoicePreference):
     section = notification
     name = "addressing_name_format"
     default = "first_last"
-    required = False
     verbose_name = _("Name format for addressing")
     choices = (
         ("first_last", "John Doe"),
         ("last_fist", "Doe, John"),
     )
+    required = True
 
 
 @person_preferences_registry.register
@@ -263,6 +268,30 @@ class SignupEnabled(BooleanPreference):
     verbose_name = _("Enable signup")
 
 
+@site_preferences_registry.register
+class InviteEnabled(BooleanPreference):
+    section = auth
+    name = "invite_enabled"
+    default = False
+    verbose_name = _("Enable invitations")
+
+
+@site_preferences_registry.register
+class InviteCodeLength(IntegerPreference):
+    section = auth
+    name = "invite_code_length"
+    default = 3
+    verbose_name = _("Length of invite code. (Default 3: abcde-acbde-abcde)")
+
+
+@site_preferences_registry.register
+class InviteCodePacketSize(IntegerPreference):
+    section = auth
+    name = "invite_code_packet_size"
+    default = 5
+    verbose_name = _("Size of packets. (Default 5: abcde)")
+
+
 @site_preferences_registry.register
 class OAuthAllowedGrants(MultipleChoicePreference):
     """Grant Flows allowed for OAuth applications."""
@@ -274,6 +303,7 @@ class OAuthAllowedGrants(MultipleChoicePreference):
     verbose_name = _("Allowed Grant Flows for OAuth applications")
     field_attribute = {"initial": []}
     choices = AbstractApplication.GRANT_TYPES
+    required = False
 
 
 @site_preferences_registry.register
@@ -287,6 +317,7 @@ class AvailableLanguages(MultipleChoicePreference):
     verbose_name = _("Available languages")
     field_attribute = {"initial": []}
     choices = settings.LANGUAGES
+    required = True
 
 
 @site_preferences_registry.register
@@ -350,6 +381,7 @@ class EditableFieldsPerson(MultipleChoicePreference):
     verbose_name = _("Fields on person model which are editable by themselves.")
     field_attribute = {"initial": []}
     choices = [(field.name, field.name) for field in Person.syncable_fields()]
+    required = False
 
 
 @site_preferences_registry.register
@@ -365,6 +397,7 @@ class SendNotificationOnPersonChange(MultipleChoicePreference):
     )
     field_attribute = {"initial": []}
     choices = [(field.name, field.name) for field in Person.syncable_fields()]
+    required = False
 
 
 @site_preferences_registry.register
@@ -375,6 +408,7 @@ class PersonChangeNotificationContact(StringPreference):
     name = "person_change_notification_contact"
     default = ""
     verbose_name = _("Contact for notification if a person changes their data")
+    required = False
 
 
 @site_preferences_registry.register
@@ -406,3 +440,13 @@ class AutoUpdatingDashboardSite(BooleanPreference):
     name = "automatically_update_dashboard_site"
     default = True
     verbose_name = _("Automatically update the dashboard and its widgets sitewide")
+
+
+@site_preferences_registry.register
+class PhoneNumberCountry(ChoicePreference):
+    section = internationalisation
+    name = "phone_number_country"
+    required = True
+    default = "GB"
+    choices = [(x.alpha_2, x.alpha_2) for x in pycountry.countries]
+    verbose_name = _("Country for phone number parsing")
