@@ -58,6 +58,12 @@ view_photo_predicate = has_person & (
 )
 rules.add_perm("core.view_photo_rule", view_photo_predicate)
 
+# View person avatar image
+view_avatar_predicate = has_person & (
+    has_global_perm("core.view_avatar") | has_object_perm("core.view_avatar") | is_current_person
+)
+rules.add_perm("core.view_avatar_rule", view_avatar_predicate)
+
 # View persons groups
 view_groups_predicate = has_person & (
     has_global_perm("core.view_person_groups")
@@ -197,7 +203,7 @@ rules.add_perm("core.change_additionalfield_rule", change_additional_field_predi
 
 # Edit additional field
 create_additional_field_predicate = has_person & (
-    has_global_perm("core.create_additionalfield") | has_object_perm("core.create_additionalfield")
+    has_global_perm("core.add_additionalfield") | has_object_perm("core.add_additionalfield")
 )
 rules.add_perm("core.create_additionalfield_rule", create_additional_field_predicate)
 
@@ -223,7 +229,7 @@ rules.add_perm("core.edit_grouptype_rule", change_group_type_predicate)
 
 # Create group type
 create_group_type_predicate = has_person & (
-    has_global_perm("core.create_grouptype") | has_object_perm("core.change_grouptype")
+    has_global_perm("core.add_grouptype") | has_object_perm("core.add_grouptype")
 )
 rules.add_perm("core.create_grouptype_rule", create_group_type_predicate)
 
@@ -242,13 +248,13 @@ rules.add_perm("core.view_grouptypes_rule", view_group_types_predicate)
 
 # Create person
 create_person_predicate = has_person & (
-    has_global_perm("core.create_person") | has_object_perm("core.create_person")
+    has_global_perm("core.add_person") | has_object_perm("core.add_person")
 )
 rules.add_perm("core.create_person_rule", create_person_predicate)
 
 # Create group
 create_group_predicate = has_person & (
-    has_global_perm("core.create_group") | has_object_perm("core.create_group")
+    has_global_perm("core.add_group") | has_object_perm("core.add_group")
 )
 rules.add_perm("core.create_group_rule", create_group_predicate)
 
@@ -319,6 +325,13 @@ rules.add_perm("core.can_register", can_register_predicate)
 can_change_password_predicate = is_site_preference_set(section="auth", pref="allow_password_change")
 rules.add_perm("core.can_change_password", can_change_password_predicate)
 
+# django-invitations
+invite_enabled_predicate = is_site_preference_set(section="auth", pref="invite_enabled")
+rules.add_perm("core.invite_enabled", invite_enabled_predicate)
+
+can_invite_predicate = has_person & has_global_perm("core.invite")
+rules.add_perm("core.can_invite", can_invite_predicate)
+
 # OAuth2 permissions
 create_oauthapplication_predicate = has_person & has_global_perm("core.add_oauthapplication")
 rules.add_perm("core.create_oauthapplication_rule", create_oauthapplication_predicate)
