@@ -1,14 +1,19 @@
+from textwrap import wrap
+
 from django.utils.translation import gettext_lazy as _
 
 import django_tables2 as tables
 from django_tables2.utils import A
 
+from .models import Person
+from .util.core_helpers import get_site_preferences
+
 
 class SchoolTermTable(tables.Table):
     """Table to list persons."""
 
     class Meta:
-        attrs = {"class": "responsive-table highlight"}
+        attrs = {"class": "highlight"}
 
     name = tables.LinkColumn("edit_school_term", args=[A("id")])
     date_start = tables.Column()
@@ -26,7 +31,7 @@ class PersonsTable(tables.Table):
     """Table to list persons."""
 
     class Meta:
-        attrs = {"class": "responsive-table highlight"}
+        attrs = {"class": "highlight"}
 
     first_name = tables.LinkColumn("person_by_id", args=[A("id")])
     last_name = tables.LinkColumn("person_by_id", args=[A("id")])
@@ -36,7 +41,7 @@ class GroupsTable(tables.Table):
     """Table to list groups."""
 
     class Meta:
-        attrs = {"class": "responsive-table highlight"}
+        attrs = {"class": "highlight"}
 
     name = tables.LinkColumn("group_by_id", args=[A("id")])
     short_name = tables.LinkColumn("group_by_id", args=[A("id")])
@@ -47,7 +52,7 @@ class AdditionalFieldsTable(tables.Table):
     """Table to list group types."""
 
     class Meta:
-        attrs = {"class": "responsive-table hightlight"}
+        attrs = {"class": "hightlight"}
 
     title = tables.LinkColumn("edit_additional_field_by_id", args=[A("id")])
     delete = tables.LinkColumn(
@@ -63,7 +68,7 @@ class GroupTypesTable(tables.Table):
     """Table to list group types."""
 
     class Meta:
-        attrs = {"class": "responsive-table highlight"}
+        attrs = {"class": "highlight"}
 
     name = tables.LinkColumn("edit_group_type_by_id", args=[A("id")])
     description = tables.LinkColumn("edit_group_type_by_id", args=[A("id")])
@@ -76,7 +81,7 @@ class DashboardWidgetTable(tables.Table):
     """Table to list dashboard widgets."""
 
     class Meta:
-        attrs = {"class": "responsive-table highlight"}
+        attrs = {"class": "highlight"}
 
     widget_name = tables.Column(accessor="pk")
     title = tables.LinkColumn("edit_dashboard_widget", args=[A("id")])
@@ -93,6 +98,33 @@ class DashboardWidgetTable(tables.Table):
         return record._meta.verbose_name
 
 
+class PersonColumn(tables.Column):
+    """Returns person object from given id."""
+
+    def render(self, value):
+        return Person.objects.get(user__id=value)
+
+
+class InvitationCodeColumn(tables.Column):
+    """Returns invitation code in a more readable format."""
+
+    def render(self, value):
+        packet_size = get_site_preferences()["auth__invite_code_packet_size"]
+        return "-".join(wrap(value, packet_size))
+
+
+class InvitationsTable(tables.Table):
+    """Table to list persons."""
+
+    email = tables.EmailColumn()
+    sent = tables.DateColumn()
+    inviter_id = PersonColumn()
+    key = InvitationCodeColumn()
+    accepted = tables.BooleanColumn(
+        yesno="check,cancel", attrs={"span": {"class": "material-icons"}}
+    )
+
+
 class PermissionDeleteColumn(tables.LinkColumn):
     """Link column with label 'Delete'."""
 
