@@ -3,7 +3,7 @@ from datetime import datetime, timedelta
 from importlib import import_module, metadata
 from itertools import groupby
 from operator import itemgetter
-from typing import Any, Callable, Optional, Sequence, Union
+from typing import Any, Callable, Dict, Optional, Sequence, Union
 from warnings import warn
 
 from django.conf import settings
@@ -13,7 +13,9 @@ from django.db.models import Model, QuerySet
 from django.http import HttpRequest
 from django.shortcuts import get_object_or_404
 from django.utils import timezone
+from django.utils.crypto import get_random_string
 from django.utils.functional import lazy
+from django.utils.module_loading import import_string
 
 from cache_memoize import cache_memoize
 
@@ -198,7 +200,7 @@ def has_person(obj: Union[HttpRequest, Model]) -> bool:
         return True
 
 
-def custom_information_processor(request: HttpRequest) -> dict:
+def custom_information_processor(request: Union[HttpRequest, None]) -> dict:
     """Provide custom information in all templates."""
     from ..models import CustomMenu
 
@@ -278,6 +280,11 @@ def queryset_rules_filter(
     return queryset.filter(pk__in=wanted_objects)
 
 
+def generate_random_code(length, packet_size) -> str:
+    """Generate random code for e.g. invitations."""
+    return get_random_string(packet_size * length).lower()
+
+
 def unread_notifications_badge(request: HttpRequest) -> int:
     """Generate badge content with the number of unread notifications."""
     return request.user.person.unread_notifications_count
@@ -316,3 +323,69 @@ def get_allowed_object_ids(request: HttpRequest, models: list) -> list:
         ]
 
     return allowed_object_ids
+
+
+def process_custom_context_processors(context_processors: list) -> Dict[str, Any]:
+    """Process custom context processors."""
+    context = {}
+    processors = tuple(import_string(path) for path in context_processors)
+    for processor in processors:
+        context.update(processor(None))
+    return context
+
+
+def create_default_celery_schedule():
+    """Create default periodic tasks in database for tasks that have a schedule defined."""
+    from celery import current_app
+    from celery.schedules import BaseSchedule, crontab, schedule, solar
+    from django_celery_beat.clockedschedule import clocked
+    from django_celery_beat.models import (
+        ClockedSchedule,
+        CrontabSchedule,
+        IntervalSchedule,
+        PeriodicTask,
+        SolarSchedule,
+    )
+
+    defined_periodic_tasks = PeriodicTask.objects.values_list("task", flat=True).all()
+
+    for name, task in current_app.tasks.items():
+        if name in defined_periodic_tasks:
+            # Task is already known in database, skip
+            continue
+
+        run_every = getattr(task, "run_every", None)
+        if not run_every:
+            # Task has no default schedule, skip
+            continue
+
+        if isinstance(run_every, (float, int, timedelta)):
+            # Schedule is defined as a raw seconds value or timedelta, convert to schedule class
+            run_every = schedule(run_every)
+        elif not isinstance(run_every, BaseSchedule):
+            raise ValueError(f"Task {name} has an invalid schedule defined.")
+
+        # Find matching django-celery-beat schedule model
+        if isinstance(run_every, clocked):
+            Schedule = ClockedSchedule
+            attr = "clocked"
+        elif isinstance(run_every, crontab):
+            Schedule = CrontabSchedule
+            attr = "crontab"
+        elif isinstance(run_every, schedule):
+            Schedule = IntervalSchedule
+            attr = "interval"
+        elif isinstance(run_every, solar):
+            Schedule = SolarSchedule
+            attr = "solar"
+        else:
+            raise ValueError(f"Task {name} has an unknown schedule class defined.")
+
+        # Get or create schedule in database
+        db_schedule = Schedule.from_schedule(run_every)
+        db_schedule.save()
+
+        # Create periodic task
+        PeriodicTask.objects.create(
+            name=f"{name} (default schedule)", task=name, **{attr: db_schedule}
+        )
